-------------------------------------------------------------------------------

-- top_32b_EDC - Module

-------------------------------------------------------------------------------

--**************************************************************************

-- 

--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"

--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR

--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION

--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION

--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS

--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,

--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE

--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY

--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE

--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR

--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF

--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS

--     FOR A PARTICULAR PURPOSE.

--

--     (c) Copyright 2003 Xilinx, Inc.

--     All rights reserved.

--

--**************************************************************************

-------------------------------------------------------------------------------

-- Filename:        top_32b_EDC.vhd

-------------------------------------------------------------------------------

-- Description:

--      Top level design for 32-bit Error Detection and Correction.

--

-------------------------------------------------------------------------------

--Structure:

--      --top_32b_EDC.vhd

--

-------------------------------------------------------------------------------

-- Owners:          Simon Tam 

-- Revision 1.0

--

--     Modification History:                                                   

--     Date     Init          Description                                

--   --------  ------ ---------------------------------------------------------    

--   9/1/2003    STAM	    Initial Release 

-------------------------------------------------------------------------------

-- Virtex-II Pro Error Detection and Correction (EDC) VHDL reference design

-- This VHDL reference design describes implementing a Error Detection and Correction (EDC) module

-- in a Virtex-II or Virtex-II Pro device. The design can detect and correct all single-bit errors,

-- and detect all double-bit errors in a 64-bit data and 8-bit parity code word. This design utilizes

-- Hamming code, a simple yet powerful solution for EDC operations. As a result, this design

-- offers exceptional performance and resource utilization.


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
-- pragma translate_off
--library UNISIM;
--use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


ENTITY EDC IS
   PORT (
      clk                     : IN std_logic;   
      resetN                  : IN std_logic;   
      encin                   : IN std_logic_vector(31 DOWNTO 0);   
      encout                  : OUT std_logic_vector(31 DOWNTO 0);   
      parity_out              : OUT std_logic_vector(6 DOWNTO 0);   
      decin                   : IN std_logic_vector(31 DOWNTO 0);   
      decout                  : OUT std_logic_vector(31 DOWNTO 0);   
      parity_in               : IN std_logic_vector(6 DOWNTO 0);   
      force_error             : IN std_logic_vector(1 DOWNTO 0);   
      error                   : OUT std_logic_vector(1 DOWNTO 0));   
END ENTITY EDC;

ARCHITECTURE translated OF EDC IS


   --reg[6:0] syndrome;
   SIGNAL syndrome                 :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL syndrome_chk             :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL syndrome_pipe            :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL enc_chkbits              :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL mask                     :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL decin_reg                :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL encin_reg                :  std_logic_vector(31 DOWNTO 0);   
   SIGNAL single_error             :  std_logic_vector(38 DOWNTO 0);   
   SIGNAL double_error             :  std_logic_vector(38 DOWNTO 0);   
   SIGNAL triple_error             :  std_logic_vector(38 DOWNTO 0);   
   SIGNAL parity_out_reg           :  std_logic_vector(6 DOWNTO 0);   
   SIGNAL parity_in_reg            :  std_logic_vector(6 DOWNTO 0);   
--   SIGNAL encout             :  std_logic_vector(31 DOWNTO 0);   
--   SIGNAL parity_out         :  std_logic_vector(6 DOWNTO 0);   
--   SIGNAL decout             :  std_logic_vector(31 DOWNTO 0);   
--   SIGNAL error              :  std_logic_vector(1 DOWNTO 0);   

BEGIN
--   encout <= encout;
--   parity_out <= parity_out;
--   decout <= decout;
--   error <= error;

   Registered_IO : PROCESS (clk, resetN)
   BEGIN
      IF (resetN = '0') THEN
         decout <= (others => '0');
         encin_reg <= (others => '0');
         decin_reg <= (others => '0');
         parity_in_reg <= (others => '0');  
         -- syndrome <= {6{1'b0}} ;
         
         
      ELSIF (clk'EVENT AND clk = '1') THEN
         encin_reg <= encin;    
         decin_reg <= decin;    
         parity_in_reg <= parity_in;    
         decout <= mask XOR decin_reg;    
         --	 syndrome <= syndrome_pipe; //add pipeline
         
         
      END IF;
   END PROCESS Registered_IO;

   -- syndrom error code
   
   -- 00 = no error/data corrected
   
   -- 01 = single bit error
   
   -- 10 = double bit error
   
   -- 11 = single check-bit error
   
   error_status : PROCESS (resetN, clk)
   BEGIN
      IF (resetN = '0') THEN
         error <= "00";    
      ELSIF (clk'EVENT AND clk = '1') THEN
         CASE syndrome(6) IS
            WHEN '0' =>
                     CASE syndrome(5 DOWNTO 0) IS
                        WHEN "000000" =>
                                 error <= "00";    --  no error 
                        WHEN OTHERS  =>
                                 error <= "10";    --  double error
                        
                     END CASE;
            -- detect multiple errors
            
            -- errors in rows101, 110 and 111 are not valid single error
            
            WHEN '1' =>
                     CASE syndrome(5 DOWNTO 3) IS
                        WHEN "101" =>
                                 error <= "11";    
                        WHEN "110" =>
                                 error <= "11";    
                        WHEN "111" =>
                                 error <= "11";    
                        WHEN OTHERS  =>
                                 error <= "01";    --  single error
                        
                     END CASE;
            WHEN OTHERS  =>
                     error <= "11";    
            
         END CASE;
      END IF;
   END PROCESS error_status;

   correction_mask : PROCESS (resetN, syndrome)
   BEGIN
      IF (resetN = '0') THEN
         mask <= (others => '0');
      ELSE
         CASE syndrome IS
            WHEN "1000011" =>
                     mask <= "00000000000000000000000000000001";    --  0
            WHEN "1000101" =>
                     mask <= "00000000000000000000000000000010";    --  1			
            WHEN "1000110" =>
                     mask <= "00000000000000000000000000000100";    --  2			
            WHEN "1000111" =>
                     mask <= "00000000000000000000000000001000";    --  3
            WHEN "1001001" =>
                     mask <= "00000000000000000000000000010000";    --  4
            WHEN "1001010" =>
                     mask <= "00000000000000000000000000100000";    --  5
            WHEN "1001011" =>
                     mask <= "00000000000000000000000001000000";    --  6
            WHEN "1001100" =>
                     mask <= "00000000000000000000000010000000";    --  7
            WHEN "1001101" =>
                     mask <= "00000000000000000000000100000000";    --  8
            WHEN "1001110" =>
                     mask <= "00000000000000000000001000000000";    --  9
            WHEN "1001111" =>
                     mask <= "00000000000000000000010000000000";    --  10
            WHEN "1010001" =>
                     mask <= "00000000000000000000100000000000";    --  11
            WHEN "1010010" =>
                     mask <= "00000000000000000001000000000000";    --  12
            WHEN "1010011" =>
                     mask <= "00000000000000000010000000000000";    --  13
            WHEN "1010100" =>
                     mask <= "00000000000000000100000000000000";    --  14
            WHEN "1010101" =>
                     mask <= "00000000000000001000000000000000";    --  15
            WHEN "1010110" =>
                     mask <= "00000000000000010000000000000000";    --  16
            WHEN "1010111" =>
                     mask <= "00000000000000100000000000000000";    --  17
            WHEN "1011000" =>
                     mask <= "00000000000001000000000000000000";    --  18
            WHEN "1011001" =>
                     mask <= "00000000000010000000000000000000";    --  19
            WHEN "1011010" =>
                     mask <= "00000000000100000000000000000000";    --  20
            WHEN "1011011" =>
                     mask <= "00000000001000000000000000000000";    --  21
            WHEN "1011100" =>
                     mask <= "00000000010000000000000000000000";    --  22
            WHEN "1011101" =>
                     mask <= "00000000100000000000000000000000";    --  23
            WHEN "1011110" =>
                     mask <= "00000001000000000000000000000000";    --  24
            WHEN "1011111" =>
                     mask <= "00000010000000000000000000000000";    --  25
            WHEN "1100001" =>
                     mask <= "00000100000000000000000000000000";    --  26
            WHEN "1100010" =>
                     mask <= "00001000000000000000000000000000";    --  27
            WHEN "1100011" =>
                     mask <= "00010000000000000000000000000000";    --  28
            WHEN "1100100" =>
                     mask <= "00100000000000000000000000000000";    --  29
            WHEN "1100101" =>
                     mask <= "01000000000000000000000000000000";    --  30
            WHEN "1100110" =>
                     mask <= "10000000000000000000000000000000";    --  31
            WHEN OTHERS  =>
                     mask <= "00000000000000000000000000000000";    
            
         END CASE;
      END IF;
   END PROCESS correction_mask;
   -- Syndrome creation
   
   -- Eight syndrome bits are created based on the 64-bit data and its associated checkbits
   
   --assign syndrome_pipe = syndrome_chk ^ parity_in_reg ;	//add pipeline
   syndrome <= syndrome_chk XOR parity_in_reg ;
   syndrome_chk(0) <= ((((((((((((((((decin_reg(0) XOR decin_reg(1)) XOR decin_reg(3)) XOR decin_reg(4)) XOR decin_reg(6)) XOR decin_reg(8)) XOR decin_reg(10)) XOR decin_reg(11)) XOR decin_reg(13)) XOR decin_reg(15)) XOR decin_reg(17)) XOR decin_reg(19)) XOR decin_reg(21)) XOR decin_reg(23)) XOR decin_reg(25)) XOR decin_reg(26)) XOR decin_reg(28)) XOR decin_reg(30) ;
   syndrome_chk(1) <= ((((((((((((((((decin_reg(0) XOR decin_reg(2)) XOR decin_reg(3)) XOR decin_reg(5)) XOR decin_reg(6)) XOR decin_reg(9)) XOR decin_reg(10)) XOR decin_reg(12)) XOR decin_reg(13)) XOR decin_reg(16)) XOR decin_reg(17)) XOR decin_reg(20)) XOR decin_reg(21)) XOR decin_reg(24)) XOR decin_reg(25)) XOR decin_reg(27)) XOR decin_reg(28)) XOR decin_reg(31) ;
   syndrome_chk(2) <= ((((((((((((((((decin_reg(1) XOR decin_reg(2)) XOR decin_reg(3)) XOR decin_reg(7)) XOR decin_reg(8)) XOR decin_reg(9)) XOR decin_reg(10)) XOR decin_reg(14)) XOR decin_reg(15)) XOR decin_reg(16)) XOR decin_reg(17)) XOR decin_reg(22)) XOR decin_reg(23)) XOR decin_reg(24)) XOR decin_reg(25)) XOR decin_reg(29)) XOR decin_reg(30)) XOR decin_reg(31) ;
   syndrome_chk(3) <= (((((((((((((decin_reg(4) XOR decin_reg(5)) XOR decin_reg(6)) XOR decin_reg(7)) XOR decin_reg(8)) XOR decin_reg(9)) XOR decin_reg(10)) XOR decin_reg(18)) XOR decin_reg(19)) XOR decin_reg(20)) XOR decin_reg(21)) XOR decin_reg(22)) XOR decin_reg(23)) XOR decin_reg(24)) XOR decin_reg(25) ;
   syndrome_chk(4) <= (((((((((((((decin_reg(11) XOR decin_reg(12)) XOR decin_reg(13)) XOR decin_reg(14)) XOR decin_reg(15)) XOR decin_reg(16)) XOR decin_reg(17)) XOR decin_reg(18)) XOR decin_reg(19)) XOR decin_reg(20)) XOR decin_reg(21)) XOR decin_reg(22)) XOR decin_reg(23)) XOR decin_reg(24)) XOR decin_reg(25) ;
   syndrome_chk(5) <= ((((decin_reg(26) XOR decin_reg(27)) XOR decin_reg(28)) XOR decin_reg(29)) XOR decin_reg(30)) XOR decin_reg(31) ;
   syndrome_chk(6) <= ((((((((((((((((((((((((((((((((((((decin_reg(0) XOR decin_reg(1)) XOR decin_reg(2)) XOR decin_reg(3)) XOR decin_reg(4)) XOR decin_reg(5)) XOR decin_reg(6)) XOR decin_reg(7)) XOR decin_reg(8)) XOR decin_reg(9)) XOR decin_reg(10)) XOR decin_reg(11)) XOR decin_reg(12)) XOR decin_reg(13)) XOR decin_reg(14)) XOR decin_reg(15)) XOR decin_reg(16)) XOR decin_reg(17)) XOR decin_reg(18)) XOR decin_reg(19)) XOR decin_reg(20)) XOR decin_reg(21)) XOR decin_reg(22)) XOR decin_reg(23)) XOR decin_reg(24)) XOR decin_reg(25)) XOR decin_reg(26)) XOR decin_reg(27)) XOR decin_reg(28)) XOR decin_reg(29)) XOR decin_reg(30)) XOR decin_reg(31)) XOR parity_in_reg(5)) XOR parity_in_reg(4)) XOR parity_in_reg(3)) XOR parity_in_reg(2)) XOR parity_in_reg(1)) XOR parity_in_reg(0) ;

   ------------------------------------------------------
   
   single_error_gen : PROCESS (clk, resetN)
   BEGIN
      IF (resetN = '0') THEN
         single_error <= "000000000000000000000000000000000000001";    
      ELSIF (clk'EVENT AND clk = '1') THEN
         single_error(0) <= single_error(38);    
         single_error(38 DOWNTO 1) <= single_error(37 DOWNTO 0);    
      END IF;
   END PROCESS single_error_gen;

   double_error_gen : PROCESS (clk, resetN)
   BEGIN
      IF (resetN = '0') THEN
         double_error <= "000000000000000000000000000000000000011";    
      ELSIF (clk'EVENT AND clk = '1') THEN
         double_error(0) <= double_error(38);    
         double_error(38 DOWNTO 1) <= double_error(37 DOWNTO 0);    
      END IF;
   END PROCESS double_error_gen;

   triple_error_gen : PROCESS (clk, resetN)
   BEGIN
      IF (resetN = '0') THEN
         triple_error <= "000000000000000000000000000000000000111";    
      ELSIF (clk'EVENT AND clk = '1') THEN
         triple_error(0) <= triple_error(38);    
         triple_error(38 DOWNTO 1) <= triple_error(37 DOWNTO 0);    
      END IF;
   END PROCESS triple_error_gen;

   Encoder : PROCESS (clk, resetN)
   BEGIN
      IF (resetN = '0') THEN
         encout <= (others => '0');
         parity_out_reg <= (others => '0');
      ELSIF (clk'EVENT AND clk = '1') THEN
         CASE force_error IS
            WHEN "00" =>
                     -- 0 error feedthrough
                     
                     encout <= encin_reg XOR "00000000000000000000000000000000";    
                     parity_out_reg <= enc_chkbits;    
            WHEN "01" =>
                     encout <= encin_reg XOR single_error(31 DOWNTO 0);    
                     parity_out_reg <= enc_chkbits XOR single_error(38 DOWNTO 32);    
            WHEN "10" =>
                     encout <= encin_reg XOR double_error(31 DOWNTO 0);    
                     parity_out_reg <= enc_chkbits XOR double_error(38 DOWNTO 32);    
            WHEN "11" =>
                     encout <= encin_reg XOR triple_error(31 DOWNTO 0);    
                     parity_out_reg <= enc_chkbits XOR triple_error(38 DOWNTO 32);    
            WHEN OTHERS  =>
                     encout <= encin_reg;    
                     parity_out_reg <= enc_chkbits;    
            
         END CASE;
      END IF;
   END PROCESS Encoder;
   parity_out <= parity_out_reg ;
   -- Encoder checkbit generator equations
   enc_chkbits(0) <= ((((((((((((((((encin_reg(0) XOR encin_reg(1)) XOR encin_reg(3)) XOR encin_reg(4)) XOR encin_reg(6)) XOR encin_reg(8)) XOR encin_reg(10)) XOR encin_reg(11)) XOR encin_reg(13)) XOR encin_reg(15)) XOR encin_reg(17)) XOR encin_reg(19)) XOR encin_reg(21)) XOR encin_reg(23)) XOR encin_reg(25)) XOR encin_reg(26)) XOR encin_reg(28)) XOR encin_reg(30) ;
   enc_chkbits(1) <= ((((((((((((((((encin_reg(0) XOR encin_reg(2)) XOR encin_reg(3)) XOR encin_reg(5)) XOR encin_reg(6)) XOR encin_reg(9)) XOR encin_reg(10)) XOR encin_reg(12)) XOR encin_reg(13)) XOR encin_reg(16)) XOR encin_reg(17)) XOR encin_reg(20)) XOR encin_reg(21)) XOR encin_reg(24)) XOR encin_reg(25)) XOR encin_reg(27)) XOR encin_reg(28)) XOR encin_reg(31) ;
   enc_chkbits(2) <= ((((((((((((((((encin_reg(1) XOR encin_reg(2)) XOR encin_reg(3)) XOR encin_reg(7)) XOR encin_reg(8)) XOR encin_reg(9)) XOR encin_reg(10)) XOR encin_reg(14)) XOR encin_reg(15)) XOR encin_reg(16)) XOR encin_reg(17)) XOR encin_reg(22)) XOR encin_reg(23)) XOR encin_reg(24)) XOR encin_reg(25)) XOR encin_reg(29)) XOR encin_reg(30)) XOR encin_reg(31) ;
   enc_chkbits(3) <= (((((((((((((encin_reg(4) XOR encin_reg(5)) XOR encin_reg(6)) XOR encin_reg(7)) XOR encin_reg(8)) XOR encin_reg(9)) XOR encin_reg(10)) XOR encin_reg(18)) XOR encin_reg(19)) XOR encin_reg(20)) XOR encin_reg(21)) XOR encin_reg(22)) XOR encin_reg(23)) XOR encin_reg(24)) XOR encin_reg(25) ;
   enc_chkbits(4) <= (((((((((((((encin_reg(11) XOR encin_reg(12)) XOR encin_reg(13)) XOR encin_reg(14)) XOR encin_reg(15)) XOR encin_reg(16)) XOR encin_reg(17)) XOR encin_reg(18)) XOR encin_reg(19)) XOR encin_reg(20)) XOR encin_reg(21)) XOR encin_reg(22)) XOR encin_reg(23)) XOR encin_reg(24)) XOR encin_reg(25) ;
   enc_chkbits(5) <= ((((encin_reg(26) XOR encin_reg(27)) XOR encin_reg(28)) XOR encin_reg(29)) XOR encin_reg(30)) XOR encin_reg(31) ;
   enc_chkbits(6) <= ((((((((((((((((((((((((((((((((((((encin_reg(0) XOR encin_reg(1)) XOR encin_reg(2)) XOR encin_reg(3)) XOR encin_reg(4)) XOR encin_reg(5)) XOR encin_reg(6)) XOR encin_reg(7)) XOR encin_reg(8)) XOR encin_reg(9)) XOR encin_reg(10)) XOR encin_reg(11)) XOR encin_reg(12)) XOR encin_reg(13)) XOR encin_reg(14)) XOR encin_reg(15)) XOR encin_reg(16)) XOR encin_reg(17)) XOR encin_reg(18)) XOR encin_reg(19)) XOR encin_reg(20)) XOR encin_reg(21)) XOR encin_reg(22)) XOR encin_reg(23)) XOR encin_reg(24)) XOR encin_reg(25)) XOR encin_reg(26)) XOR encin_reg(27)) XOR encin_reg(28)) XOR encin_reg(29)) XOR encin_reg(30)) XOR encin_reg(31)) XOR enc_chkbits(5)) XOR enc_chkbits(4)) XOR enc_chkbits(3)) XOR enc_chkbits(2)) XOR enc_chkbits(1)) XOR enc_chkbits(0) ;

END ARCHITECTURE translated;
