-------------------------------------------------------------------------------
-- top - Module
-------------------------------------------------------------------------------
--**************************************************************************
-- 
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--**************************************************************************
-------------------------------------------------------------------------------
-- Filename:        hamming64_classic_pipe1.vhd
-------------------------------------------------------------------------------
-- Description:
--      Top level design for 64-bit Error Detection and Correction.
--
-------------------------------------------------------------------------------
--Structure:
--      --top.vhd
--
-------------------------------------------------------------------------------
-- Owners:          Simon Tam 
-- Revision 1.0
--
--     Modification History:                                                   
--     Date     Init          Description                                
--   --------  ------ ---------------------------------------------------------
--   2/26/2003   STAM     Initial Release 
--   12/11/2003  STAM     1 stage pipeline
-------------------------------------------------------------------------------

-- Virtex-II Pro Error Detection and Correction (EDC) VHDL reference design
-- This VHDL reference design describes implementing a Error Detection and Correction (EDC) module
-- in a Virtex-II or Virtex-II Pro device. The design can detect and correct all single-bit errors,
-- and detect all double-bit errors in a 64-bit data and 8-bit parity code word. This design utilizes
-- Hamming code, a simple yet powerful solution for EDC operations. As a result, this design
-- offers exceptional performance and resource utilization.

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
-- pragma translate_off
LIBRARY UNISIM;
USE UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

ENTITY hamming64_classic_pipe1 is
    PORT ( 	clk_in : in std_logic;
           	resetN : in std_logic;
	    	encin : in std_logic_vector (63 downto 0);
	    	encout : out std_logic_vector (63 downto 0);
	    	parity_out : out std_logic_vector (7 downto 0);
           	decin : in std_logic_vector(63 downto 0);
           	decout : out std_logic_vector(63 downto 0);
	    	parity_in : in std_logic_vector (7 downto 0);
	    	force_error : in std_logic_vector (1 downto 0);
           	error : out std_logic_vector (1 downto 0);
			lock  : out std_logic
			  );
END hamming64_classic_pipe1;

ARCHITECTURE Behavioral of hamming64_classic_pipe1 is

component BUFG 
  port (
  	I   : in std_logic;
        O    : out std_logic
	); 
end component;

component DCM

    generic ( 
             DLL_FREQUENCY_MODE : string := "LOW";
             DUTY_CYCLE_CORRECTION : boolean := TRUE;
             STARTUP_WAIT : boolean := FALSE
            );  
--

    port ( CLKIN     : in  std_logic;
           CLKFB     : in  std_logic;
           DSSEN     : in  std_logic;
           PSINCDEC  : in  std_logic;
           PSEN      : in  std_logic;
           PSCLK     : in  std_logic;
           RST       : in  std_logic;
           CLK0      : out std_logic;
           CLK90     : out std_logic;
           CLK180    : out std_logic;
           CLK270    : out std_logic;
           CLK2X     : out std_logic;
           CLK2X180  : out std_logic;
           CLKDV     : out std_logic;
           CLKFX     : out std_logic;
           CLKFX180  : out std_logic;
           LOCKED    : out std_logic;
           PSDONE    : out std_logic;
           STATUS    : out std_logic_vector(7 downto 0)
          );
end component;

SIGNAL syndrome_chk, syndrome, syndrome_reg, enc_chkbits, enc_chkbits_reg :  std_logic_vector (7 downto 0);
SIGNAL mask, decin_reg, decin_reg1, encin_reg, encin_reg1 : std_logic_vector (63 downto 0);
SIGNAL single_error, double_error, triple_error :  std_logic_vector (71 downto 0);
SIGNAL parity_out_reg, parity_in_reg :  std_logic_vector (7 downto 0);
SIGNAL GND : std_logic;
SIGNAL CLK0_W: std_logic;
SIGNAL CLK1X_W, clk, reset: std_logic;

BEGIN

GND <= '0';
clk <= CLK1X_W;
reset <= not resetN;

U_DCM: DCM
  port map (
            CLKIN =>    CLK_IN,
            CLKFB =>    CLK1X_W,
            DSSEN =>    GND,
            PSINCDEC => GND,
            PSEN =>     GND,
            PSCLK =>    GND,
            RST =>      reset,
            CLK0 =>     CLK0_W,
            LOCKED =>   LOCK
	   ); 

-- BUFG Instantiation
U_BUFG: BUFG
  port map (
  	    I => CLK0_W,
            O => CLK1X_W
  	   );


Registered_IO : PROCESS (clk, resetN)
begin
	if resetN = '0' then
			decout <= (others => '0');
			encin_reg <= (others => '0');
			decin_reg <= (others => '0');
			parity_in_reg <= (others => '0');
	elsif clk = '1' and clk'event then
			encin_reg <= encin;
			decin_reg <= decin;
			parity_in_reg <= parity_in;
			decout <= mask xor decin_reg1;
	end if;

END PROCESS;

-- syndrom error code
-- 00 = no error/data corrected
-- 01 = single bit error
-- 10 = double bit error
-- 11 = single check-bit error

error_status : PROCESS (resetN, clk, syndrome)
begin
	if resetN = '0' then
		error <= (others => '0');
	elsif clk = '1' and clk'event then
		case syndrome(7) is
			when '0' => 
				case syndrome(6 downto 0) is
					when "0000000" => error <= "00"; -- no error 
					when others    => error <= "10"; -- double error
				end case;
			when '1' => 
				case syndrome(6 downto 3) is
					when "1001" => error <= "11"; -- detect multiple errors
					when "1010" => error <= "11";
					when "1011" => error <= "11";
					when "1100" => error <= "11";
					when "1101" => error <= "11";
					when "1110" => error <= "11";
					when "1111" => error <= "11";																																		
					when others => error <= "01"; -- single error
				end case;
			when others => error <= "11";
		end case;
	end if;
END PROCESS;

correction_mask : PROCESS (resetN, syndrome)
begin
	if resetN = '0' then
		mask <= (others => '0');
	else
		case syndrome is
			when "10000011" => mask <= X"0000000000000001"; -- 0
			when "10000101" => mask <= X"0000000000000002"; -- 1			
			when "10000110" => mask <= X"0000000000000004"; -- 2			
			when "10000111" => mask <= X"0000000000000008"; -- 3
			when "10001001" => mask <= X"0000000000000010"; -- 4
			when "10001010" => mask <= X"0000000000000020"; -- 5
			when "10001011" => mask <= X"0000000000000040"; -- 6
			when "10001100" => mask <= X"0000000000000080"; -- 7
			when "10001101" => mask <= X"0000000000000100"; -- 8
			when "10001110" => mask <= X"0000000000000200"; -- 9

			when "10001111" => mask <= X"0000000000000400"; -- 10
			when "10010001" => mask <= X"0000000000000800"; -- 11
			when "10010010" => mask <= X"0000000000001000"; -- 12
			when "10010011" => mask <= X"0000000000002000"; -- 13
			when "10010100" => mask <= X"0000000000004000"; -- 14
			when "10010101" => mask <= X"0000000000008000"; -- 15
			when "10010110" => mask <= X"0000000000010000"; -- 16
			when "10010111" => mask <= X"0000000000020000"; -- 17
			when "10011000" => mask <= X"0000000000040000"; -- 18
			when "10011001" => mask <= X"0000000000080000"; -- 19

			when "10011010" => mask <= X"0000000000100000"; -- 20
			when "10011011" => mask <= X"0000000000200000"; -- 21
			when "10011100" => mask <= X"0000000000400000"; -- 22
			when "10011101" => mask <= X"0000000000800000"; -- 23
			when "10011110" => mask <= X"0000000001000000"; -- 24
			when "10011111" => mask <= X"0000000002000000"; -- 25
			when "10100001" => mask <= X"0000000004000000"; -- 26
			when "10100010" => mask <= X"0000000008000000"; -- 27
			when "10100011" => mask <= X"0000000010000000"; -- 28
			when "10100100" => mask <= X"0000000020000000"; -- 29

			when "10100101" => mask <= X"0000000040000000"; -- 30
			when "10100110" => mask <= X"0000000080000000"; -- 31
			when "10100111" => mask <= X"0000000100000000"; -- 32
			when "10101000" => mask <= X"0000000200000000"; -- 33
			when "10101001" => mask <= X"0000000400000000"; -- 34
			when "10101010" => mask <= X"0000000800000000"; -- 35
			when "10101011" => mask <= X"0000001000000000"; -- 36
			when "10101100" => mask <= X"0000002000000000"; -- 37
			when "10101101" => mask <= X"0000004000000000"; -- 38
			when "10101110" => mask <= X"0000008000000000"; -- 39

			when "10101111" => mask <= X"0000010000000000"; -- 40
			when "10110000" => mask <= X"0000020000000000"; -- 41
			when "10110001" => mask <= X"0000040000000000"; -- 42
			when "10110010" => mask <= X"0000080000000000"; -- 43
			when "10110011" => mask <= X"0000100000000000"; -- 44
			when "10110100" => mask <= X"0000200000000000"; -- 45
			when "10110101" => mask <= X"0000400000000000"; -- 46
			when "10110110" => mask <= X"0000800000000000"; -- 47
			when "10110111" => mask <= X"0001000000000000"; -- 48
			when "10111000" => mask <= X"0002000000000000"; -- 49

			when "10111001" => mask <= X"0004000000000000"; -- 50
			when "10111010" => mask <= X"0008000000000000"; -- 51
			when "10111011" => mask <= X"0010000000000000"; -- 52
			when "10111100" => mask <= X"0020000000000000"; -- 53
			when "10111101" => mask <= X"0040000000000000"; -- 54
			when "10111110" => mask <= X"0080000000000000"; -- 55
			when "10111111" => mask <= X"0100000000000000"; -- 56
			when "11000001" => mask <= X"0200000000000000"; -- 57
			when "11000010" => mask <= X"0400000000000000"; -- 58
			when "11000011" => mask <= X"0800000000000000"; -- 59

			when "11000100" => mask <= X"1000000000000000"; -- 60
			when "11000101" => mask <= X"2000000000000000"; -- 61
			when "11000110" => mask <= X"4000000000000000"; -- 62
			when "11000111" => mask <= X"8000000000000000"; -- 63	
			when  others 	=> mask <= X"0000000000000000";
		end case;
	end if;
END PROCESS;

-- Syndrome creation
-- Eight syndrome bits are created based on the 64-bit data and its associated checkbits

-- pipelined
dec_pipeline : PROCESS (resetN, clk)
BEGIN
	if resetN = '0' then
		syndrome_reg <= (others => '0');
		decin_reg1 <= (others => '0');
	elsif clk = '1' and clk'event then
		syndrome_reg <= syndrome_chk xor parity_in_reg;
		decin_reg1 <= decin_reg;
    end if;
END PROCESS;

syndrome <= syndrome_reg;

-- non-pipelined
--syndrome <= syndrome_chk xor parity_in_reg;

syndrome_chk(0) 
<= decin_reg(0) xor decin_reg(1) xor decin_reg(3) xor decin_reg(4) xor decin_reg(6) xor decin_reg(8) xor decin_reg(10) xor decin_reg(11)
xor decin_reg(13) xor decin_reg(15) xor decin_reg(17) xor decin_reg(19) xor decin_reg(21) xor decin_reg(23) xor decin_reg(25) xor decin_reg(26)
xor decin_reg(28) xor decin_reg(30) xor decin_reg(32) xor decin_reg(34) xor decin_reg(36) xor decin_reg(38) xor decin_reg(40) xor decin_reg(42)
xor decin_reg(44) xor decin_reg(46) xor decin_reg(48) xor decin_reg(50) xor decin_reg(52) xor decin_reg(54) xor decin_reg(56) xor decin_reg(57)
xor decin_reg(59) xor decin_reg(61) xor decin_reg(63);

syndrome_chk(1) 
<= decin_reg(0) xor decin_reg(2) xor decin_reg(3) xor decin_reg(5) xor decin_reg(6) xor decin_reg(9) xor decin_reg(10) xor decin_reg(12)
xor decin_reg(13) xor decin_reg(16) xor decin_reg(17) xor decin_reg(20) xor decin_reg(21) xor decin_reg(24) xor decin_reg(25) xor decin_reg(27)
xor decin_reg(28) xor decin_reg(31) xor decin_reg(32) xor decin_reg(35) xor decin_reg(36) xor decin_reg(39) xor decin_reg(40) xor decin_reg(43)
xor decin_reg(44) xor decin_reg(47) xor decin_reg(48) xor decin_reg(51) xor decin_reg(52) xor decin_reg(55) xor decin_reg(56) xor decin_reg(58)
xor decin_reg(59) xor decin_reg(62) xor decin_reg(63);

syndrome_chk(2) 
<= decin_reg(1) xor decin_reg(2) xor decin_reg(3) xor decin_reg(7) xor decin_reg(8) xor decin_reg(9) xor decin_reg(10) xor decin_reg(14)
xor decin_reg(15) xor decin_reg(16) xor decin_reg(17) xor decin_reg(22) xor decin_reg(23) xor decin_reg(24) xor decin_reg(25) xor decin_reg(29)
xor decin_reg(30) xor decin_reg(31) xor decin_reg(32) xor decin_reg(37) xor decin_reg(38) xor decin_reg(39) xor decin_reg(40) xor decin_reg(45)
xor decin_reg(46) xor decin_reg(47) xor decin_reg(48) xor decin_reg(53) xor decin_reg(54) xor decin_reg(55) xor decin_reg(56) xor decin_reg(60)
xor decin_reg(61) xor decin_reg(62) xor decin_reg(63);

syndrome_chk(3) 
<= decin_reg(4) xor decin_reg(5) xor decin_reg(6) xor decin_reg(7) xor decin_reg(8) xor decin_reg(9) xor decin_reg(10) xor decin_reg(18)
xor decin_reg(19) xor decin_reg(20) xor decin_reg(21) xor decin_reg(22) xor decin_reg(23) xor decin_reg(24) xor decin_reg(25) xor decin_reg(33)
xor decin_reg(34) xor decin_reg(35) xor decin_reg(36) xor decin_reg(37) xor decin_reg(38) xor decin_reg(39) xor decin_reg(40) xor decin_reg(49)
xor decin_reg(50) xor decin_reg(51) xor decin_reg(52) xor decin_reg(53) xor decin_reg(54) xor decin_reg(55) xor decin_reg(56);

syndrome_chk(4) 
<=  decin_reg(11) xor decin_reg(12) xor decin_reg(13) xor decin_reg(14) xor decin_reg(15) xor decin_reg(16) xor decin_reg(17) xor decin_reg(18)
xor decin_reg(19) xor decin_reg(20) xor decin_reg(21) xor decin_reg(22) xor decin_reg(23) xor decin_reg(24) xor decin_reg(25) xor decin_reg(41)
xor decin_reg(42) xor decin_reg(43) xor decin_reg(44) xor decin_reg(45) xor decin_reg(46) xor decin_reg(47) xor decin_reg(48) xor decin_reg(49)
xor decin_reg(50) xor decin_reg(51) xor decin_reg(52) xor decin_reg(53) xor decin_reg(54) xor decin_reg(55) xor decin_reg(56);

syndrome_chk(5)
<=  decin_reg(26) xor decin_reg(27) xor decin_reg(28) xor decin_reg(29) xor decin_reg(30) xor decin_reg(31) xor decin_reg(32) xor decin_reg(33)
xor decin_reg(34) xor decin_reg(35) xor decin_reg(36) xor decin_reg(37) xor decin_reg(38) xor decin_reg(39) xor decin_reg(40) xor decin_reg(41) 
xor decin_reg(42) xor decin_reg(43) xor decin_reg(44) xor decin_reg(45) xor decin_reg(46) xor decin_reg(47) xor decin_reg(48) xor decin_reg(49) 
xor decin_reg(50) xor decin_reg(51) xor decin_reg(52) xor decin_reg(53) xor decin_reg(54) xor decin_reg(55) xor decin_reg(56);

syndrome_chk(6)
<= decin_reg(57) xor decin_reg(58) xor decin_reg(59) xor decin_reg(60) xor decin_reg(61) xor decin_reg(62) xor decin_reg(63);

syndrome_chk(7)
<=  decin_reg(0)  xor decin_reg(1)  xor decin_reg(2)  xor decin_reg(3)  xor decin_reg(4)  xor decin_reg(5)  xor decin_reg(6)  xor decin_reg(7)  
xor decin_reg(8)  xor decin_reg(9)  xor decin_reg(10) xor decin_reg(11) xor decin_reg(12) xor decin_reg(13) xor decin_reg(14) xor decin_reg(15) 
xor decin_reg(16) xor decin_reg(17) xor decin_reg(18) xor decin_reg(19) xor decin_reg(20) xor decin_reg(21) xor decin_reg(22) xor decin_reg(23) 
xor decin_reg(24) xor decin_reg(25) xor decin_reg(26) xor decin_reg(27) xor decin_reg(28) xor decin_reg(29) xor decin_reg(30) xor decin_reg(31) 
xor decin_reg(32) xor decin_reg(33) xor decin_reg(34) xor decin_reg(35) xor decin_reg(36) xor decin_reg(37) xor decin_reg(38) xor decin_reg(39) 
xor decin_reg(40) xor decin_reg(41) xor decin_reg(42) xor decin_reg(43) xor decin_reg(44) xor decin_reg(45) xor decin_reg(46) xor decin_reg(47) 
xor decin_reg(48) xor decin_reg(49) xor decin_reg(50) xor decin_reg(51) xor decin_reg(52) xor decin_reg(53) xor decin_reg(54) xor decin_reg(55) 
xor decin_reg(56) xor decin_reg(57) xor decin_reg(58) xor decin_reg(59) xor decin_reg(60) xor decin_reg(61) xor decin_reg(62) xor decin_reg(63)
xor parity_in_reg(6) xor parity_in_reg(5) xor parity_in_reg(4) xor parity_in_reg(3) xor parity_in_reg(2) xor parity_in_reg(1) xor parity_in_reg(0);
------------------------------------------------------

single_error_gen : PROCESS (clk, resetN)
 begin
   if resetN ='0' then
		single_error <= X"000000000000000001";
   elsif CLK='1' and CLK'event then
			single_error(0) <= single_error(71);
	         	single_error(71 downto 1) <= single_error(70 downto 0);
	end if;
END PROCESS;

double_error_gen : PROCESS (clk, resetN)
 begin
   if resetN ='0' then
--		double_error <= "0000000000000000000000000000000100000000000000000000000000000001";
		double_error <= X"000000000000000003";
   elsif CLK='1' and CLK'event then
			double_error(0) <= double_error(71);
	         	double_error(71 downto 1) <= double_error(70 downto 0);
	end if;
END PROCESS;

triple_error_gen : PROCESS (clk, resetN)
 begin
   if resetN ='0' then
		triple_error <= X"000000000000000007";
   elsif CLK='1' and CLK'event then
			triple_error(0) <= triple_error(71);
	         	triple_error(71 downto 1) <= triple_error(70 downto 0);
	end if;
END PROCESS;

Encoder : PROCESS(clk, resetN)
  begin
   if resetN ='0' then
		encout <= (others => '0');
		parity_out_reg <= (others => '0');
   elsif CLK='1' and CLK'event then
   	case force_error is 
      when  "00" =>   -- 0 error feedthrough
			encout <= encin_reg1 xor x"0000000000000000";
			parity_out_reg <= enc_chkbits_reg;

      when  "01" =>  -- 1 bit error walk-thru
			encout <= encin_reg1 xor single_error(63 downto 0);
			parity_out_reg <= enc_chkbits_reg xor single_error (71 downto 64);

      when  "10" =>   -- 2-bit error walk-thru
			encout <= encin_reg1 xor double_error(63 downto 0);
			parity_out_reg <= enc_chkbits_reg xor double_error (71 downto 64);

      when "11" =>   
			encout <= encin_reg1 xor triple_error(63 downto 0);
			parity_out_reg <= enc_chkbits_reg xor triple_error (71 downto 64);
			
	  when others =>
			encout <= encin_reg1;
			parity_out_reg <= enc_chkbits_reg;

   	end case;
      	
	end if;
END PROCESS;

parity_out <= parity_out_reg;

-- encoder piplined
enc_pipeline : PROCESS(clk, resetN)
  begin
   if resetN ='0' then
		enc_chkbits_reg <= (others => '0');
		encin_reg1 <= (others => '0');
   elsif CLK='1' and CLK'event then
		enc_chkbits_reg <= enc_chkbits; -- pipeline
		encin_reg1 <= encin_reg; -- pipeline
   	
	end if;

end process;

-- Encoder checkbit generator equations

enc_chkbits(0) 
<= encin_reg(0) xor encin_reg(1) xor encin_reg(3) xor encin_reg(4) xor encin_reg(6) xor encin_reg(8) xor encin_reg(10) xor encin_reg(11)
xor encin_reg(13) xor encin_reg(15) xor encin_reg(17) xor encin_reg(19) xor encin_reg(21) xor encin_reg(23) xor encin_reg(25) xor encin_reg(26)
xor encin_reg(28) xor encin_reg(30) xor encin_reg(32) xor encin_reg(34) xor encin_reg(36) xor encin_reg(38) xor encin_reg(40) xor encin_reg(42)
xor encin_reg(44) xor encin_reg(46) xor encin_reg(48) xor encin_reg(50) xor encin_reg(52) xor encin_reg(54) xor encin_reg(56) xor encin_reg(57)
xor encin_reg(59) xor encin_reg(61) xor encin_reg(63);

enc_chkbits(1) 
<= encin_reg(0) xor encin_reg(2) xor encin_reg(3) xor encin_reg(5) xor encin_reg(6) xor encin_reg(9) xor encin_reg(10) xor encin_reg(12)
xor encin_reg(13) xor encin_reg(16) xor encin_reg(17) xor encin_reg(20) xor encin_reg(21) xor encin_reg(24) xor encin_reg(25) xor encin_reg(27)
xor encin_reg(28) xor encin_reg(31) xor encin_reg(32) xor encin_reg(35) xor encin_reg(36) xor encin_reg(39) xor encin_reg(40) xor encin_reg(43)
xor encin_reg(44) xor encin_reg(47) xor encin_reg(48) xor encin_reg(51) xor encin_reg(52) xor encin_reg(55) xor encin_reg(56) xor encin_reg(58)
xor encin_reg(59) xor encin_reg(62) xor encin_reg(63);

enc_chkbits(2) 
<= encin_reg(1) xor encin_reg(2) xor encin_reg(3) xor encin_reg(7) xor encin_reg(8) xor encin_reg(9) xor encin_reg(10) xor encin_reg(14)
xor encin_reg(15) xor encin_reg(16) xor encin_reg(17) xor encin_reg(22) xor encin_reg(23) xor encin_reg(24) xor encin_reg(25) xor encin_reg(29)
xor encin_reg(30) xor encin_reg(31) xor encin_reg(32) xor encin_reg(37) xor encin_reg(38) xor encin_reg(39) xor encin_reg(40) xor encin_reg(45)
xor encin_reg(46) xor encin_reg(47) xor encin_reg(48) xor encin_reg(53) xor encin_reg(54) xor encin_reg(55) xor encin_reg(56) xor encin_reg(60)
xor encin_reg(61) xor encin_reg(62) xor encin_reg(63);

enc_chkbits(3) 
<= encin_reg(4) xor encin_reg(5) xor encin_reg(6) xor encin_reg(7) xor encin_reg(8) xor encin_reg(9) xor encin_reg(10) xor encin_reg(18)
xor encin_reg(19) xor encin_reg(20) xor encin_reg(21) xor encin_reg(22) xor encin_reg(23) xor encin_reg(24) xor encin_reg(25) xor encin_reg(33)
xor encin_reg(34) xor encin_reg(35) xor encin_reg(36) xor encin_reg(37) xor encin_reg(38) xor encin_reg(39) xor encin_reg(40) xor encin_reg(49)
xor encin_reg(50) xor encin_reg(51) xor encin_reg(52) xor encin_reg(53) xor encin_reg(54) xor encin_reg(55) xor encin_reg(56);

enc_chkbits(4) 
<=  encin_reg(11) xor encin_reg(12) xor encin_reg(13) xor encin_reg(14) xor encin_reg(15) xor encin_reg(16) xor encin_reg(17) xor encin_reg(18)
xor encin_reg(19) xor encin_reg(20) xor encin_reg(21) xor encin_reg(22) xor encin_reg(23) xor encin_reg(24) xor encin_reg(25) xor encin_reg(41)
xor encin_reg(42) xor encin_reg(43) xor encin_reg(44) xor encin_reg(45) xor encin_reg(46) xor encin_reg(47) xor encin_reg(48) xor encin_reg(49)
xor encin_reg(50) xor encin_reg(51) xor encin_reg(52) xor encin_reg(53) xor encin_reg(54) xor encin_reg(55) xor encin_reg(56);

enc_chkbits(5)
<=  encin_reg(26) xor encin_reg(27) xor encin_reg(28) xor encin_reg(29) xor encin_reg(30) xor encin_reg(31) xor encin_reg(32) xor encin_reg(33)
xor encin_reg(34) xor encin_reg(35) xor encin_reg(36) xor encin_reg(37) xor encin_reg(38) xor encin_reg(39) xor encin_reg(40) xor encin_reg(41) 
xor encin_reg(42) xor encin_reg(43) xor encin_reg(44) xor encin_reg(45) xor encin_reg(46) xor encin_reg(47) xor encin_reg(48) xor encin_reg(49) 
xor encin_reg(50) xor encin_reg(51) xor encin_reg(52) xor encin_reg(53) xor encin_reg(54) xor encin_reg(55) xor encin_reg(56);

enc_chkbits(6)
<= encin_reg(57) xor encin_reg(58) xor encin_reg(59) xor encin_reg(60) xor encin_reg(61) xor encin_reg(62) xor encin_reg(63);

enc_chkbits(7)
<=  encin_reg(0)  xor encin_reg(1)  xor encin_reg(2)  xor encin_reg(3)  xor encin_reg(4)  xor encin_reg(5)  xor encin_reg(6)  xor encin_reg(7)  
xor encin_reg(8)  xor encin_reg(9)  xor encin_reg(10) xor encin_reg(11) xor encin_reg(12) xor encin_reg(13) xor encin_reg(14) xor encin_reg(15) 
xor encin_reg(16) xor encin_reg(17) xor encin_reg(18) xor encin_reg(19) xor encin_reg(20) xor encin_reg(21) xor encin_reg(22) xor encin_reg(23) 
xor encin_reg(24) xor encin_reg(25) xor encin_reg(26) xor encin_reg(27) xor encin_reg(28) xor encin_reg(29) xor encin_reg(30) xor encin_reg(31) 
xor encin_reg(32) xor encin_reg(33) xor encin_reg(34) xor encin_reg(35) xor encin_reg(36) xor encin_reg(37) xor encin_reg(38) xor encin_reg(39) 
xor encin_reg(40) xor encin_reg(41) xor encin_reg(42) xor encin_reg(43) xor encin_reg(44) xor encin_reg(45) xor encin_reg(46) xor encin_reg(47) 
xor encin_reg(48) xor encin_reg(49) xor encin_reg(50) xor encin_reg(51) xor encin_reg(52) xor encin_reg(53) xor encin_reg(54) xor encin_reg(55) 
xor encin_reg(56) xor encin_reg(57) xor encin_reg(58) xor encin_reg(59) xor encin_reg(60) xor encin_reg(61) xor encin_reg(62) xor encin_reg(63)
xor enc_chkbits(6) xor enc_chkbits(5) xor enc_chkbits(4) xor enc_chkbits(3) xor enc_chkbits(2) xor enc_chkbits(1) xor enc_chkbits(0);

END BEHAVIORAL;
