//-----------------------------------------------------------------------------
// top_32b_EDC - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2003 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        top_32b_EDC.v
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 32-bit Error Detection and Correction.
//
//-----------------------------------------------------------------------------
//Structure:
//      --top_32b_EDC.v
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------    
//   9/1/2003    STAM	    Initial Release 
//-----------------------------------------------------------------------------
// Virtex-II Pro Error Detection and Correction (EDC) VHDL reference design
// This VHDL reference design describes implementing a Error Detection and Correction (EDC) module
// in a Virtex-II or Virtex-II Pro device. The design can detect and correct all single-bit errors,
// and detect all double-bit errors in a 64-bit data and 8-bit parity code word. This design utilizes
// Hamming code, a simple yet powerful solution for EDC operations. As a result, this design
// offers exceptional performance and resource utilization.
// pragma translate_off
// pragma translate_on

`timescale 1ns/100ps

module EDC (clk, resetN, encin, encout, parity_out, decin, decout, parity_in, force_error, error);

   input clk; 
   input resetN; 
   input[31:0] encin; 
   output[31:0] encout; 
   reg[31:0] encout;
   output[6:0] parity_out; 
   wire[6:0] parity_out;
   input[31:0] decin; 
   output[31:0] decout; 
   reg[31:0] decout;
   input[6:0] parity_in; 
   input[1:0] force_error; 
   output[1:0] error; 
   reg[1:0] error;
   //reg[6:0] syndrome;
   wire [6:0] syndrome;
   wire[6:0] syndrome_chk; 
   wire[6:0] syndrome_pipe; 
   wire[6:0] enc_chkbits; 
   reg[31:0] mask; 
   reg[31:0] decin_reg; 
   reg[31:0] encin_reg; 
   reg[38:0] single_error; 
   reg[38:0] double_error; 
   reg[38:0] triple_error; 
   reg[6:0] parity_out_reg; 
   reg[6:0] parity_in_reg; 

   always @(posedge clk or negedge resetN)
   begin : Registered_IO
      if (resetN == 1'b0)
      begin
         decout <= {32{1'b0}} ; 
         encin_reg <= {32{1'b0}} ; 
         decin_reg <= {32{1'b0}} ; 
         parity_in_reg <= {7{1'b0}} ;
 		// syndrome <= {6{1'b0}} ;
      end
      else
      begin
         encin_reg <= encin ; 
         decin_reg <= decin ; 
         parity_in_reg <= parity_in ; 
         decout <= mask ^ decin_reg ; 
	//	 syndrome <= syndrome_pipe; //add pipeline
      end 
   end 

   // syndrom error code
   // 00 = no error/data corrected
   // 01 = single bit error
   // 10 = double bit error
   // 11 = single check-bit error
   always @(negedge resetN or posedge clk)
   begin : error_status
      if (resetN == 1'b0)
      begin
         error <= {2{1'b0}} ; 
      end
      else
      begin
         case (syndrome[6])
            1'b0 :
                     begin
                        case (syndrome[5:0])
                           6'b000000 :
                                    begin
                                       error <= 2'b00 ; // no error 
                                    end
                           default :
                                    begin
                                       error <= 2'b10 ; // double error
                                    end
                        endcase 
                     end
 // detect multiple errors
 // errors in rows101, 110 and 111 are not valid single error
            1'b1 :
                     begin
                        case (syndrome[5:3])
                           3'b101 :
                                    begin
                                       error <= 2'b11 ;
                                    end
                           3'b110 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           3'b111 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           default :
                                    begin
                                       error <= 2'b01 ; // single error
                                    end
                        endcase 
                     end
            default :
                     begin
                        error <= 2'b11 ; 
                     end
         endcase 
      end 
   end 

   always @(resetN or syndrome)
   begin : correction_mask
      if (resetN == 1'b0)
      begin
         mask <= {32{1'b0}} ; 
      end
      else
      begin
         case (syndrome)
            7'b1000011 :
                     begin
                        mask <= 32'h00000001 ; // 0
                     end
            7'b1000101 :
                     begin
                        mask <= 32'h00000002 ; // 1			
                     end
            7'b1000110 :
                     begin
                        mask <= 32'h00000004 ; // 2			
                     end
            7'b1000111 :
                     begin
                        mask <= 32'h00000008 ; // 3
                     end
            7'b1001001 :
                     begin
                        mask <= 32'h00000010 ; // 4
                     end
            7'b1001010 :
                     begin
                        mask <= 32'h00000020 ; // 5
                     end
            7'b1001011 :
                     begin
                        mask <= 32'h00000040 ; // 6
                     end
            7'b1001100 :
                     begin
                        mask <= 32'h00000080 ; // 7
                     end
            7'b1001101 :
                     begin
                        mask <= 32'h00000100 ; // 8
                     end
            7'b1001110 :
                     begin
                        mask <= 32'h00000200 ; // 9
                     end
            7'b1001111 :
                     begin
                        mask <= 32'h00000400 ; // 10
                     end
            7'b1010001 :
                     begin
                        mask <= 32'h00000800 ; // 11
                     end
            7'b1010010 :
                     begin
                        mask <= 32'h00001000 ; // 12
                     end
            7'b1010011 :
                     begin
                        mask <= 32'h00002000 ; // 13
                     end
            7'b1010100 :
                     begin
                        mask <= 32'h00004000 ; // 14
                     end
            7'b1010101 :
                     begin
                        mask <= 32'h00008000 ; // 15
                     end
            7'b1010110 :
                     begin
                        mask <= 32'h00010000 ; // 16
                     end
            7'b1010111 :
                     begin
                        mask <= 32'h00020000 ; // 17
                     end
            7'b1011000 :
                     begin
                        mask <= 32'h00040000 ; // 18
                     end
            7'b1011001 :
                     begin
                        mask <= 32'h00080000 ; // 19
                     end
            7'b1011010 :
                     begin
                        mask <= 32'h00100000 ; // 20
                     end
            7'b1011011 :
                     begin
                        mask <= 32'h00200000 ; // 21
                     end
            7'b1011100 :
                     begin
                        mask <= 32'h00400000 ; // 22
                     end
            7'b1011101 :
                     begin
                        mask <= 32'h00800000 ; // 23
                     end
            7'b1011110 :
                     begin
                        mask <= 32'h01000000 ; // 24
                     end
            7'b1011111 :
                     begin
                        mask <= 32'h02000000 ; // 25
                     end
            7'b1100001 :
                     begin
                        mask <= 32'h04000000 ; // 26
                     end
            7'b1100010 :
                     begin
                        mask <= 32'h08000000 ; // 27
                     end
            7'b1100011 :
                     begin
                        mask <= 32'h10000000 ; // 28
                     end
            7'b1100100 :
                     begin
                        mask <= 32'h20000000 ; // 29
                     end
            7'b1100101 :
                     begin
                        mask <= 32'h40000000 ; // 30
                     end
            7'b1100110 :
                     begin
                        mask <= 32'h80000000 ; // 31
                     end
            default :
                     begin
                        mask <= 32'h00000000 ; 
                     end
         endcase 
      end 
   end 
   // Syndrome creation
   // Eight syndrome bits are created based on the 64-bit data and its associated checkbits
   
   //assign syndrome_pipe = syndrome_chk ^ parity_in_reg ;	//add pipeline
   assign syndrome = syndrome_chk ^ parity_in_reg ;
   
   assign syndrome_chk[0] = decin_reg[0] ^ decin_reg[1] ^ decin_reg[3] ^ decin_reg[4] ^ decin_reg[6] ^ decin_reg[8] ^ decin_reg[10] ^ decin_reg[11] ^ decin_reg[13] ^ decin_reg[15] ^ decin_reg[17] ^ decin_reg[19] ^ decin_reg[21] ^ decin_reg[23] ^ decin_reg[25] ^ decin_reg[26] ^ decin_reg[28] ^ decin_reg[30];
   assign syndrome_chk[1] = decin_reg[0] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[31];
   assign syndrome_chk[2] = decin_reg[1] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31];
   assign syndrome_chk[3] = decin_reg[4] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25];
   assign syndrome_chk[4] = decin_reg[11] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ;
   assign syndrome_chk[5] = decin_reg[26] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31] ;
   assign syndrome_chk[6] = decin_reg[0] ^ decin_reg[1] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[4] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[11] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[26] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31] ^ parity_in_reg[5] ^ parity_in_reg[4] ^ parity_in_reg[3] ^ parity_in_reg[2] ^ parity_in_reg[1] ^ parity_in_reg[0] ;

   //----------------------------------------------------
   always @(posedge clk or negedge resetN)
   begin : single_error_gen
      if (resetN == 1'b0)
      begin
         single_error <= 39'h0000000001 ; 
      end
      else
      begin
         single_error[0] <= single_error[38] ; 
         single_error[38:1] <= single_error[37:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : double_error_gen
      if (resetN == 1'b0)
      begin
         double_error <= 39'h0000000003 ; 
      end
      else
      begin
         double_error[0] <= double_error[38] ; 
         double_error[38:1] <= double_error[37:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : triple_error_gen
      if (resetN == 1'b0)
      begin
         triple_error <= 39'h0000000007 ; 
      end
      else
      begin
         triple_error[0] <= triple_error[38] ; 
         triple_error[38:1] <= triple_error[37:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : Encoder
      if (resetN == 1'b0)
      begin
         encout <= {32{1'b0}} ; 
         parity_out_reg <= {7{1'b0}} ; 
      end
      else
      begin
         case (force_error)
            2'b00 :
                     begin
                        // 0 error feedthrough
                        encout <= encin_reg ^ 32'h00000000 ; 
                        parity_out_reg <= enc_chkbits ; 
                     end
            2'b01 :
                     begin
                        // 1 bit error walk-thru
                        encout <= encin_reg ^ single_error[31:0] ; 
                        parity_out_reg <= enc_chkbits ^ single_error[38:32] ; 
                     end
            2'b10 :
                     begin
                        // 2-bit error walk-thru
                        encout <= encin_reg ^ double_error[31:0] ; 
                        parity_out_reg <= enc_chkbits ^ double_error[38:32] ; 
                     end
            2'b11 :
                     begin
                        encout <= encin_reg ^ triple_error[31:0] ; 
                        parity_out_reg <= enc_chkbits ^ triple_error[38:32] ; 
                     end
            default :
                     begin
                        encout <= encin_reg ; 
                        parity_out_reg <= enc_chkbits ; 
                     end
         endcase 
      end 
   end 
   assign parity_out = parity_out_reg ;
   // Encoder checkbit generator equations
   assign enc_chkbits[0] = encin_reg[0] ^ encin_reg[1] ^ encin_reg[3] ^ encin_reg[4] ^ encin_reg[6] ^ encin_reg[8] ^ encin_reg[10] ^ encin_reg[11] ^ encin_reg[13] ^ encin_reg[15] ^ encin_reg[17] ^ encin_reg[19] ^ encin_reg[21] ^ encin_reg[23] ^ encin_reg[25] ^ encin_reg[26] ^ encin_reg[28] ^ encin_reg[30];
   assign enc_chkbits[1] = encin_reg[0] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[31] ;
   assign enc_chkbits[2] = encin_reg[1] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ;
   assign enc_chkbits[3] = encin_reg[4] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25];
   assign enc_chkbits[4] = encin_reg[11] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25];
   assign enc_chkbits[5] = encin_reg[26] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ;
   assign enc_chkbits[6] = encin_reg[0] ^ encin_reg[1] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[4] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[11] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[26] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ^ enc_chkbits[5] ^ enc_chkbits[4] ^ enc_chkbits[3] ^ enc_chkbits[2] ^ enc_chkbits[1] ^ enc_chkbits[0] ;
endmodule
