//-----------------------------------------------------------------------------
// top - Module
//-----------------------------------------------------------------------------
//**************************************************************************
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2003 Xilinx, Inc.
//     All rights reserved.
//
//**************************************************************************
//-----------------------------------------------------------------------------
// Filename:        top.vhd
//-----------------------------------------------------------------------------
// Description:
//      Top level design for 64-bit Error Detection and Correction.
//
//-----------------------------------------------------------------------------
//Structure:
//      --top.vhd
//
//-----------------------------------------------------------------------------
// Owners:          Simon Tam 
// Revision 1.0
//
//     Modification History:                                                   
//     Date     Init          Description                                
//   --------  ------ ---------------------------------------------------------
//   2/26/2003   STAM     Initial Release 
//-----------------------------------------------------------------------------
// Virtex-II Pro Error Detection and Correction (EDC) VHDL reference design
// This VHDL reference design describes implementing a Error Detection and Correction (EDC) module
// in a Virtex-II or Virtex-II Pro device. The design can detect and correct all single-bit errors,
// and detect all double-bit errors in a 64-bit data and 8-bit parity code word. This design utilizes
// Hamming code, a simple yet powerful solution for EDC operations. As a result, this design
// offers exceptional performance and resource utilization.
// pragma translate_off
// pragma translate_on

`timescale 1ns/100ps

module EDC (clk, resetN, encin, encout, parity_out, decin, decout, parity_in, force_error, error);

   input clk; 
   input resetN; 
   input[63:0] encin; 
   output[63:0] encout; 
   reg[63:0] encout;
   output[7:0] parity_out; 
   wire[7:0] parity_out;
   input[63:0] decin; 
   output[63:0] decout; 
   reg[63:0] decout;
   input[7:0] parity_in; 
   input[1:0] force_error; 
   output[1:0] error; 
   reg[1:0] error;

   wire[7:0] syndrome_chk; 
   wire[7:0] syndrome; 
   wire[7:0] enc_chkbits; 
   reg[63:0] mask; 
   reg[63:0] decin_reg; 
   reg[63:0] encin_reg; 
   reg[71:0] single_error; 
   reg[71:0] double_error; 
   reg[71:0] triple_error; 
   reg[7:0] parity_out_reg; 
   reg[7:0] parity_in_reg; 

   always @(posedge clk or negedge resetN)
   begin : Registered_IO
      if (resetN == 1'b0)
      begin
         decout <= {64{1'b0}} ; 
         encin_reg <= {64{1'b0}} ; 
         decin_reg <= {64{1'b0}} ; 
         parity_in_reg <= {8{1'b0}} ; 
      end
      else
      begin
         encin_reg <= encin ; 
         decin_reg <= decin ; 
         parity_in_reg <= parity_in ; 
         decout <= mask ^ decin_reg ; 
      end 
   end 

   // syndrom error code
   // 00 = no error/data corrected
   // 01 = single bit error
   // 10 = double bit error
   // 11 = single check-bit error
   always @(negedge resetN or posedge clk)
   begin : error_status
      if (resetN == 1'b0)
      begin
         error <= {2{1'b0}} ; 
      end
      else
      begin
         case (syndrome[7])
            1'b0 :
                     begin
                        case (syndrome[6:0])
                           7'b0000000 :
                                    begin
                                       error <= 2'b00 ; // no error 
                                    end
                           default :
                                    begin
                                       error <= 2'b10 ; // double error
                                    end
                        endcase 
                     end
            1'b1 :
                     begin
                        case (syndrome[6:3])
                           4'b1001 :
                                    begin
                                       error <= 2'b11 ; // detect multiple errors
                                    end
                           4'b1010 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           4'b1011 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           4'b1100 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           4'b1101 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           4'b1110 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           4'b1111 :
                                    begin
                                       error <= 2'b11 ; 
                                    end
                           default :
                                    begin
                                       error <= 2'b01 ; // single error
                                    end
                        endcase 
                     end
            default :
                     begin
                        error <= 2'b11 ; 
                     end
         endcase 
      end 
   end 

   always @(resetN or syndrome)
   begin : correction_mask
      if (resetN == 1'b0)
      begin
         mask <= {64{1'b0}} ; 
      end
      else
      begin
         case (syndrome)
            8'b10000011 :
                     begin
                        mask <= 64'h0000000000000001 ; // 0
                     end
            8'b10000101 :
                     begin
                        mask <= 64'h0000000000000002 ; // 1			
                     end
            8'b10000110 :
                     begin
                        mask <= 64'h0000000000000004 ; // 2			
                     end
            8'b10000111 :
                     begin
                        mask <= 64'h0000000000000008 ; // 3
                     end
            8'b10001001 :
                     begin
                        mask <= 64'h0000000000000010 ; // 4
                     end
            8'b10001010 :
                     begin
                        mask <= 64'h0000000000000020 ; // 5
                     end
            8'b10001011 :
                     begin
                        mask <= 64'h0000000000000040 ; // 6
                     end
            8'b10001100 :
                     begin
                        mask <= 64'h0000000000000080 ; // 7
                     end
            8'b10001101 :
                     begin
                        mask <= 64'h0000000000000100 ; // 8
                     end
            8'b10001110 :
                     begin
                        mask <= 64'h0000000000000200 ; // 9
                     end
            8'b10001111 :
                     begin
                        mask <= 64'h0000000000000400 ; // 10
                     end
            8'b10010001 :
                     begin
                        mask <= 64'h0000000000000800 ; // 11
                     end
            8'b10010010 :
                     begin
                        mask <= 64'h0000000000001000 ; // 12
                     end
            8'b10010011 :
                     begin
                        mask <= 64'h0000000000002000 ; // 13
                     end
            8'b10010100 :
                     begin
                        mask <= 64'h0000000000004000 ; // 14
                     end
            8'b10010101 :
                     begin
                        mask <= 64'h0000000000008000 ; // 15
                     end
            8'b10010110 :
                     begin
                        mask <= 64'h0000000000010000 ; // 16
                     end
            8'b10010111 :
                     begin
                        mask <= 64'h0000000000020000 ; // 17
                     end
            8'b10011000 :
                     begin
                        mask <= 64'h0000000000040000 ; // 18
                     end
            8'b10011001 :
                     begin
                        mask <= 64'h0000000000080000 ; // 19
                     end
            8'b10011010 :
                     begin
                        mask <= 64'h0000000000100000 ; // 20
                     end
            8'b10011011 :
                     begin
                        mask <= 64'h0000000000200000 ; // 21
                     end
            8'b10011100 :
                     begin
                        mask <= 64'h0000000000400000 ; // 22
                     end
            8'b10011101 :
                     begin
                        mask <= 64'h0000000000800000 ; // 23
                     end
            8'b10011110 :
                     begin
                        mask <= 64'h0000000001000000 ; // 24
                     end
            8'b10011111 :
                     begin
                        mask <= 64'h0000000002000000 ; // 25
                     end
            8'b10100001 :
                     begin
                        mask <= 64'h0000000004000000 ; // 26
                     end
            8'b10100010 :
                     begin
                        mask <= 64'h0000000008000000 ; // 27
                     end
            8'b10100011 :
                     begin
                        mask <= 64'h0000000010000000 ; // 28
                     end
            8'b10100100 :
                     begin
                        mask <= 64'h0000000020000000 ; // 29
                     end
            8'b10100101 :
                     begin
                        mask <= 64'h0000000040000000 ; // 30
                     end
            8'b10100110 :
                     begin
                        mask <= 64'h0000000080000000 ; // 31
                     end
            8'b10100111 :
                     begin
                        mask <= 64'h0000000100000000 ; // 32
                     end
            8'b10101000 :
                     begin
                        mask <= 64'h0000000200000000 ; // 33
                     end
            8'b10101001 :
                     begin
                        mask <= 64'h0000000400000000 ; // 34
                     end
            8'b10101010 :
                     begin
                        mask <= 64'h0000000800000000 ; // 35
                     end
            8'b10101011 :
                     begin
                        mask <= 64'h0000001000000000 ; // 36
                     end
            8'b10101100 :
                     begin
                        mask <= 64'h0000002000000000 ; // 37
                     end
            8'b10101101 :
                     begin
                        mask <= 64'h0000004000000000 ; // 38
                     end
            8'b10101110 :
                     begin
                        mask <= 64'h0000008000000000 ; // 39
                     end
            8'b10101111 :
                     begin
                        mask <= 64'h0000010000000000 ; // 40
                     end
            8'b10110000 :
                     begin
                        mask <= 64'h0000020000000000 ; // 41
                     end
            8'b10110001 :
                     begin
                        mask <= 64'h0000040000000000 ; // 42
                     end
            8'b10110010 :
                     begin
                        mask <= 64'h0000080000000000 ; // 43
                     end
            8'b10110011 :
                     begin
                        mask <= 64'h0000100000000000 ; // 44
                     end
            8'b10110100 :
                     begin
                        mask <= 64'h0000200000000000 ; // 45
                     end
            8'b10110101 :
                     begin
                        mask <= 64'h0000400000000000 ; // 46
                     end
            8'b10110110 :
                     begin
                        mask <= 64'h0000800000000000 ; // 47
                     end
            8'b10110111 :
                     begin
                        mask <= 64'h0001000000000000 ; // 48
                     end
            8'b10111000 :
                     begin
                        mask <= 64'h0002000000000000 ; // 49
                     end
            8'b10111001 :
                     begin
                        mask <= 64'h0004000000000000 ; // 50
                     end
            8'b10111010 :
                     begin
                        mask <= 64'h0008000000000000 ; // 51
                     end
            8'b10111011 :
                     begin
                        mask <= 64'h0010000000000000 ; // 52
                     end
            8'b10111100 :
                     begin
                        mask <= 64'h0020000000000000 ; // 53
                     end
            8'b10111101 :
                     begin
                        mask <= 64'h0040000000000000 ; // 54
                     end
            8'b10111110 :
                     begin
                        mask <= 64'h0080000000000000 ; // 55
                     end
            8'b10111111 :
                     begin
                        mask <= 64'h0100000000000000 ; // 56
                     end
            8'b11000001 :
                     begin
                        mask <= 64'h0200000000000000 ; // 57
                     end
            8'b11000010 :
                     begin
                        mask <= 64'h0400000000000000 ; // 58
                     end
            8'b11000011 :
                     begin
                        mask <= 64'h0800000000000000 ; // 59
                     end
            8'b11000100 :
                     begin
                        mask <= 64'h1000000000000000 ; // 60
                     end
            8'b11000101 :
                     begin
                        mask <= 64'h2000000000000000 ; // 61
                     end
            8'b11000110 :
                     begin
                        mask <= 64'h4000000000000000 ; // 62
                     end
            8'b11000111 :
                     begin
                        mask <= 64'h8000000000000000 ; // 63	
                     end
            default :
                     begin
                        mask <= 64'h0000000000000000 ; 
                     end
         endcase 
      end 
   end 
   // Syndrome creation
   // Eight syndrome bits are created based on the 64-bit data and its associated checkbits
   assign syndrome = syndrome_chk ^ parity_in_reg ;
   assign syndrome_chk[0] = decin_reg[0] ^ decin_reg[1] ^ decin_reg[3] ^ decin_reg[4] ^ decin_reg[6] ^ decin_reg[8] ^ decin_reg[10] ^ decin_reg[11] ^ decin_reg[13] ^ decin_reg[15] ^ decin_reg[17] ^ decin_reg[19] ^ decin_reg[21] ^ decin_reg[23] ^ decin_reg[25] ^ decin_reg[26] ^ decin_reg[28] ^ decin_reg[30] ^ decin_reg[32] ^ decin_reg[34] ^ decin_reg[36] ^ decin_reg[38] ^ decin_reg[40] ^ decin_reg[42] ^ decin_reg[44] ^ decin_reg[46] ^ decin_reg[48] ^ decin_reg[50] ^ decin_reg[52] ^ decin_reg[54] ^ decin_reg[56] ^ decin_reg[57] ^ decin_reg[59] ^ decin_reg[61] ^ decin_reg[63] ;
   assign syndrome_chk[1] = decin_reg[0] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[31] ^ decin_reg[32] ^ decin_reg[35] ^ decin_reg[36] ^ decin_reg[39] ^ decin_reg[40] ^ decin_reg[43] ^ decin_reg[44] ^ decin_reg[47] ^ decin_reg[48] ^ decin_reg[51] ^ decin_reg[52] ^ decin_reg[55] ^ decin_reg[56] ^ decin_reg[58] ^ decin_reg[59] ^ decin_reg[62] ^ decin_reg[63] ;
   assign syndrome_chk[2] = decin_reg[1] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31] ^ decin_reg[32] ^ decin_reg[37] ^ decin_reg[38] ^ decin_reg[39] ^ decin_reg[40] ^ decin_reg[45] ^ decin_reg[46] ^ decin_reg[47] ^ decin_reg[48] ^ decin_reg[53] ^ decin_reg[54] ^ decin_reg[55] ^ decin_reg[56] ^ decin_reg[60] ^ decin_reg[61] ^ decin_reg[62] ^ decin_reg[63] ;
   assign syndrome_chk[3] = decin_reg[4] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[33] ^ decin_reg[34] ^ decin_reg[35] ^ decin_reg[36] ^ decin_reg[37] ^ decin_reg[38] ^ decin_reg[39] ^ decin_reg[40] ^ decin_reg[49] ^ decin_reg[50] ^ decin_reg[51] ^ decin_reg[52] ^ decin_reg[53] ^ decin_reg[54] ^ decin_reg[55] ^ decin_reg[56] ;
   assign syndrome_chk[4] = decin_reg[11] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[41] ^ decin_reg[42] ^ decin_reg[43] ^ decin_reg[44] ^ decin_reg[45] ^ decin_reg[46] ^ decin_reg[47] ^ decin_reg[48] ^ decin_reg[49] ^ decin_reg[50] ^ decin_reg[51] ^ decin_reg[52] ^ decin_reg[53] ^ decin_reg[54] ^ decin_reg[55] ^ decin_reg[56] ;
   assign syndrome_chk[5] = decin_reg[26] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31] ^ decin_reg[32] ^ decin_reg[33] ^ decin_reg[34] ^ decin_reg[35] ^ decin_reg[36] ^ decin_reg[37] ^ decin_reg[38] ^ decin_reg[39] ^ decin_reg[40] ^ decin_reg[41] ^ decin_reg[42] ^ decin_reg[43] ^ decin_reg[44] ^ decin_reg[45] ^ decin_reg[46] ^ decin_reg[47] ^ decin_reg[48] ^ decin_reg[49] ^ decin_reg[50] ^ decin_reg[51] ^ decin_reg[52] ^ decin_reg[53] ^ decin_reg[54] ^ decin_reg[55] ^ decin_reg[56] ;
   assign syndrome_chk[6] = decin_reg[57] ^ decin_reg[58] ^ decin_reg[59] ^ decin_reg[60] ^ decin_reg[61] ^ decin_reg[62] ^ decin_reg[63] ;
   assign syndrome_chk[7] = decin_reg[0] ^ decin_reg[1] ^ decin_reg[2] ^ decin_reg[3] ^ decin_reg[4] ^ decin_reg[5] ^ decin_reg[6] ^ decin_reg[7] ^ decin_reg[8] ^ decin_reg[9] ^ decin_reg[10] ^ decin_reg[11] ^ decin_reg[12] ^ decin_reg[13] ^ decin_reg[14] ^ decin_reg[15] ^ decin_reg[16] ^ decin_reg[17] ^ decin_reg[18] ^ decin_reg[19] ^ decin_reg[20] ^ decin_reg[21] ^ decin_reg[22] ^ decin_reg[23] ^ decin_reg[24] ^ decin_reg[25] ^ decin_reg[26] ^ decin_reg[27] ^ decin_reg[28] ^ decin_reg[29] ^ decin_reg[30] ^ decin_reg[31] ^ decin_reg[32] ^ decin_reg[33] ^ decin_reg[34] ^ decin_reg[35] ^ decin_reg[36] ^ decin_reg[37] ^ decin_reg[38] ^ decin_reg[39] ^ decin_reg[40] ^ decin_reg[41] ^ decin_reg[42] ^ decin_reg[43] ^ decin_reg[44] ^ decin_reg[45] ^ decin_reg[46] ^ decin_reg[47] ^ decin_reg[48] ^ decin_reg[49] ^ decin_reg[50] ^ decin_reg[51] ^ decin_reg[52] ^ decin_reg[53] ^ decin_reg[54] ^ decin_reg[55] ^ decin_reg[56] ^ decin_reg[57] ^ decin_reg[58] ^ decin_reg[59] ^ decin_reg[60] ^ decin_reg[61] ^ decin_reg[62] ^ decin_reg[63] ^ parity_in_reg[6] ^ parity_in_reg[5] ^ parity_in_reg[4] ^ parity_in_reg[3] ^ parity_in_reg[2] ^ parity_in_reg[1] ^ parity_in_reg[0] ;

   //----------------------------------------------------
   always @(posedge clk or negedge resetN)
   begin : single_error_gen
      if (resetN == 1'b0)
      begin
         single_error <= 72'h000000000000000001 ; 
      end
      else
      begin
         single_error[0] <= single_error[71] ; 
         single_error[71:1] <= single_error[70:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : double_error_gen
      if (resetN == 1'b0)
      begin
         double_error <= 72'h000000000000000003 ; 
      end
      else
      begin
         double_error[0] <= double_error[71] ; 
         double_error[71:1] <= double_error[70:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : triple_error_gen
      if (resetN == 1'b0)
      begin
         triple_error <= 72'h000000000000000007 ; 
      end
      else
      begin
         triple_error[0] <= triple_error[71] ; 
         triple_error[71:1] <= triple_error[70:0] ; 
      end 
   end 

   always @(posedge clk or negedge resetN)
   begin : Encoder
      if (resetN == 1'b0)
      begin
         encout <= {64{1'b0}} ; 
         parity_out_reg <= {8{1'b0}} ; 
      end
      else
      begin
         case (force_error)
            2'b00 :
                     begin
                        // 0 error feedthrough
                        encout <= encin_reg ^ 64'h0000000000000000 ; 
                        parity_out_reg <= enc_chkbits ; 
                     end
            2'b01 :
                     begin
                        // 1 bit error walk-thru
                        encout <= encin_reg ^ single_error[63:0] ; 
                        parity_out_reg <= enc_chkbits ^ single_error[71:64] ; 
                     end
            2'b10 :
                     begin
                        // 2-bit error walk-thru
                        encout <= encin_reg ^ double_error[63:0] ; 
                        parity_out_reg <= enc_chkbits ^ double_error[71:64] ; 
                     end
            2'b11 :
                     begin
                        encout <= encin_reg ^ triple_error[63:0] ; 
                        parity_out_reg <= enc_chkbits ^ triple_error[71:64] ; 
                     end
            default :
                     begin
                        encout <= encin_reg ; 
                        parity_out_reg <= enc_chkbits ; 
                     end
         endcase 
      end 
   end 
   assign parity_out = parity_out_reg ;
   // Encoder checkbit generator equations
   assign enc_chkbits[0] = encin_reg[0] ^ encin_reg[1] ^ encin_reg[3] ^ encin_reg[4] ^ encin_reg[6] ^ encin_reg[8] ^ encin_reg[10] ^ encin_reg[11] ^ encin_reg[13] ^ encin_reg[15] ^ encin_reg[17] ^ encin_reg[19] ^ encin_reg[21] ^ encin_reg[23] ^ encin_reg[25] ^ encin_reg[26] ^ encin_reg[28] ^ encin_reg[30] ^ encin_reg[32] ^ encin_reg[34] ^ encin_reg[36] ^ encin_reg[38] ^ encin_reg[40] ^ encin_reg[42] ^ encin_reg[44] ^ encin_reg[46] ^ encin_reg[48] ^ encin_reg[50] ^ encin_reg[52] ^ encin_reg[54] ^ encin_reg[56] ^ encin_reg[57] ^ encin_reg[59] ^ encin_reg[61] ^ encin_reg[63] ;
   assign enc_chkbits[1] = encin_reg[0] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[31] ^ encin_reg[32] ^ encin_reg[35] ^ encin_reg[36] ^ encin_reg[39] ^ encin_reg[40] ^ encin_reg[43] ^ encin_reg[44] ^ encin_reg[47] ^ encin_reg[48] ^ encin_reg[51] ^ encin_reg[52] ^ encin_reg[55] ^ encin_reg[56] ^ encin_reg[58] ^ encin_reg[59] ^ encin_reg[62] ^ encin_reg[63] ;
   assign enc_chkbits[2] = encin_reg[1] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ^ encin_reg[32] ^ encin_reg[37] ^ encin_reg[38] ^ encin_reg[39] ^ encin_reg[40] ^ encin_reg[45] ^ encin_reg[46] ^ encin_reg[47] ^ encin_reg[48] ^ encin_reg[53] ^ encin_reg[54] ^ encin_reg[55] ^ encin_reg[56] ^ encin_reg[60] ^ encin_reg[61] ^ encin_reg[62] ^ encin_reg[63] ;
   assign enc_chkbits[3] = encin_reg[4] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[33] ^ encin_reg[34] ^ encin_reg[35] ^ encin_reg[36] ^ encin_reg[37] ^ encin_reg[38] ^ encin_reg[39] ^ encin_reg[40] ^ encin_reg[49] ^ encin_reg[50] ^ encin_reg[51] ^ encin_reg[52] ^ encin_reg[53] ^ encin_reg[54] ^ encin_reg[55] ^ encin_reg[56] ;
   assign enc_chkbits[4] = encin_reg[11] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[41] ^ encin_reg[42] ^ encin_reg[43] ^ encin_reg[44] ^ encin_reg[45] ^ encin_reg[46] ^ encin_reg[47] ^ encin_reg[48] ^ encin_reg[49] ^ encin_reg[50] ^ encin_reg[51] ^ encin_reg[52] ^ encin_reg[53] ^ encin_reg[54] ^ encin_reg[55] ^ encin_reg[56] ;
   assign enc_chkbits[5] = encin_reg[26] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ^ encin_reg[32] ^ encin_reg[33] ^ encin_reg[34] ^ encin_reg[35] ^ encin_reg[36] ^ encin_reg[37] ^ encin_reg[38] ^ encin_reg[39] ^ encin_reg[40] ^ encin_reg[41] ^ encin_reg[42] ^ encin_reg[43] ^ encin_reg[44] ^ encin_reg[45] ^ encin_reg[46] ^ encin_reg[47] ^ encin_reg[48] ^ encin_reg[49] ^ encin_reg[50] ^ encin_reg[51] ^ encin_reg[52] ^ encin_reg[53] ^ encin_reg[54] ^ encin_reg[55] ^ encin_reg[56] ;
   assign enc_chkbits[6] = encin_reg[57] ^ encin_reg[58] ^ encin_reg[59] ^ encin_reg[60] ^ encin_reg[61] ^ encin_reg[62] ^ encin_reg[63] ;
   assign enc_chkbits[7] = encin_reg[0] ^ encin_reg[1] ^ encin_reg[2] ^ encin_reg[3] ^ encin_reg[4] ^ encin_reg[5] ^ encin_reg[6] ^ encin_reg[7] ^ encin_reg[8] ^ encin_reg[9] ^ encin_reg[10] ^ encin_reg[11] ^ encin_reg[12] ^ encin_reg[13] ^ encin_reg[14] ^ encin_reg[15] ^ encin_reg[16] ^ encin_reg[17] ^ encin_reg[18] ^ encin_reg[19] ^ encin_reg[20] ^ encin_reg[21] ^ encin_reg[22] ^ encin_reg[23] ^ encin_reg[24] ^ encin_reg[25] ^ encin_reg[26] ^ encin_reg[27] ^ encin_reg[28] ^ encin_reg[29] ^ encin_reg[30] ^ encin_reg[31] ^ encin_reg[32] ^ encin_reg[33] ^ encin_reg[34] ^ encin_reg[35] ^ encin_reg[36] ^ encin_reg[37] ^ encin_reg[38] ^ encin_reg[39] ^ encin_reg[40] ^ encin_reg[41] ^ encin_reg[42] ^ encin_reg[43] ^ encin_reg[44] ^ encin_reg[45] ^ encin_reg[46] ^ encin_reg[47] ^ encin_reg[48] ^ encin_reg[49] ^ encin_reg[50] ^ encin_reg[51] ^ encin_reg[52] ^ encin_reg[53] ^ encin_reg[54] ^ encin_reg[55] ^ encin_reg[56] ^ encin_reg[57] ^ encin_reg[58] ^ encin_reg[59] ^ encin_reg[60] ^ encin_reg[61] ^ encin_reg[62] ^ encin_reg[63] ^ enc_chkbits[6] ^ enc_chkbits[5] ^ enc_chkbits[4] ^ enc_chkbits[3] ^ enc_chkbits[2] ^ enc_chkbits[1] ^ enc_chkbits[0] ;
endmodule
